;****************************************************************************
;*                     GNU Hell on Earth   ver 0.16 Beta                    *
;*                    A 3D engine for the TI 92 calculator                  *
;*                     Copyright (C) 1998  Mathieu Lacage                   *
;****************************************************************************

   include "equates.h"
   include "macros.h"
   include "tios.h"
   include "flib.h"


 ifeq OS-Plus_shell
  include "92plus.h"
  include "util.h"
 endif

 ifeq mode-test_mode
;   include "flib.h"
   include "hexlib.h"
 endif
 ifeq mode-debug_mode
   include "ghell.sdb"
 endif

 xdef    _main
 xdef    _comment


_main:

   jsr       flib::clr_scr
   Setfont   mediumfont
   DispStr   40,30,<m1>,black
   DispStr   70,40,<m2>,black
   Setfont   smallfont
   DispStr   60,50,<m3>,black
   Setfont   mediumfont
   DispStr   10,100,<m4>,black
   jsr       flib::idle_loop

   bsr     init
 ifeq mode-test_mode
   move.l  #0,counter
 endif

engine_loop:

 ifeq mode-test_mode
   add.l   #1,counter
 endif

   bsr     key_test                    ;
test_esc:                              ;
   move.b  #$FF,($60001B)              ;    test on keys and update...
   move.w  #%1011111111,($600018)      ;    the esc key is treated 
   nop                                 ;    as a separate case...
   nop                                 ;
   nop                                 ;
   nop                                 ;
   nop                                 ;
   nop                                 ;
   move.b  ($60001B),D0                ;
   btst    #6,D0                       ;
   beq     exit                        ;

 ifne mode-debug_mode
   bsr      swap_screen       ; The double buffering system. First swap
 endif

   bsr      zap_screen        ; screens then clear them

   move.l   #1,D0             ;         
   move.w   view_ang,D1       ;
   add.w    #width/2,D1       ;  adjust the ang variable and check
if_1:                         ;  it is in the right boudary
   cmp      #ang_360,D1       ;
   bcs      \end_if           ;
   sub      #ang_360,D1       ;
\end_if:                      ;

iter_1:

if_2:                     ;
   tst      D1            ;  check if ang is in the right low boudary
   bpl      \end_if       ;
\then:                    ;
   add      #ang_360,D1   ;
\end_if:                  ;

 include "engine.asm"         ; calculate the image based on the current
                              ; position
render:
 include "render.asm"

 include "draw.asm"

end:
   add      #1,D0             ;
   sub      #1,D1             ; adjust variables to calculate the new
   cmp      #width+1,D0       ; sline then relaunch the engine on this
   beq      engine_loop       ; new sline
   bra      iter_1            ;


;***********************
int_mask          dc.l 0
exit:                                   ; this is executed if and only if esc
 ifne mode-debug_mode
   move.w   plane_1_ptr_hard,($600010)  ; was hit. restore the screen
 endif
   move.l   int_mask,d0                 ; and
   trap     #1                          ; restore the old interrupt mask
 ifne mode-debug_mode
   move     handle_number,-(A7)     ;
   jsr      tios::HeapFree          ; restore the handle
   lea      2(A7),A7                ;
 endif

 ifeq mode-test_mode
   move.l   counter,d0
   movem.l  zeroes,d1-d2
   move.l   #8-1,d4
   jsr      hexlib::put_hex
   jsr      flib::idle_loop
   jsr      flib::idle_loop
 endif

   jsr      flib::clr_scr
   Setfont  1
   DispStr  10,10,<m5>,4
   DispStr  30,20,<m6>,4
   DispStr  30,30,<m7>,4
   DispStr  30,40,<m8>,4
   DispStr  10,60,<m9>,4
   DispStr  30,70,<m10>,4
   jsr      flib::idle_loop
   jsr      flib::idle_loop
   rts
;************************



;***********************
 ifeq OS-Fargo
memory_error = 670
alloc_error:
   move.w   #memory_error,-(a7)     ; there was not enough mem to run doom
   jsr      tios::ERD_dialog        ; exits with a message
   add.l    #2,a7
   rts
 endif

 ifeq OS-Plus_shell
alloc_error:
   rts
 endif
;***********************


 include "hell_sub.asm"        ; a set of subroutines

 ifeq world_size_mode-size_small
 include "wor_1024.asm"
 endif
 ifeq world_size_mode-size_medium
 include "wor_2048.asm"
 endif
 ifeq world_size_mode-size_big
 include "wor_4096.asm"
 endif

 ifeq wall_texturing-on
  ifeq wall_tex_size-small
 include "tex_16.asm"       ; wall textures
  endif
  ifeq wall_tex_size-medium
 include "tex_32.asm"       ; wall textures
  endif
  ifeq wall_tex_size-big
 include "tex_64.asm"       ; wall textures
  endif
 endif

 ifeq mode-test_mode
counter           dc.l 0
 endif

plane_ptr         dc.l 0

tan_128_ptr       dc.l 0
inv_tan_128_ptr   dc.l 0
tan_64_ptr        dc.l 0
inv_tan_64_ptr    dc.l 0
inv_cos_128_ptr   dc.l 0
inv_sin_128_ptr   dc.l 0
move_x_ptr        dc.l 0
move_y_ptr        dc.l 0
fish_ptr          dc.l 0

tables:
 incbin  "stuff2.bin"
fish:
 incbin  "fish.bin"
move_x:
 incbin  "move_x.bin"
move_y:
 incbin  "move_y.bin"


m1    dc.b "GNU Hell on earth v 0.16b",0
m2    dc.b "A GNU 3D engine",0
m3    dc.b "copyright (c) 1998 Mathieu Lacage",0
m4    dc.b "To learn about GNU, read readme.txt",0
m5    dc.b "Thanks to:",0
m6    dc.b "Patrick Davidson",0
m7    dc.b "Bryan Rabeler",0
m8    dc.b "My first beta testers",0
m9    dc.b "Special thanks to:",0
m10   dc.b "Gareth",0

my      dc.b "test",0

_comment:
          dc.b "GNU Hell on Earth v 0.16 Beta",0

 end







   

   
     
