;****************************************************************************
;*                     GNU Hell on Earth   ver 0.16 Beta                    *
;*                    A 3D engine for the TI 92 calculator                  *
;*                     Copyright (C) 1998  Mathieu Lacage                   *
;****************************************************************************

; a0: pointer to world
; a1: pointer to a table ( used to calculate the first intersection )
; a3: pointer to a table ( _step: used to calculate successive intersections )
; a4: pointer to a table ( use to calculate distance to wall )
; a6: pointer to the right entry in tables pointed to by a1,a3 and a4

; d0.b: counter: sline number beeing processed
; d1.w: counter: sline angle beeing processed
; d2.w: xb / yb
; d3.w: xi / yi
; d4.w: x
; d5.w: a temporary register used a little everywhere
; d6.w: x_step / y_step
; d7.w: y

; a2, a5 and a7 are not modified...

; please, note that all comment here are done assuming that world_size
; is equal to size_1

intersection_x_constant:

   move.l   tan_128_ptr,A1
   move.l   tan_64_ptr,A3
   move.l   inv_cos_128_ptr,A4

if_x_90:
   cmp      #ang_90,D1           ; if ang<90
   bhi      end_if_x_90          ;
then_x_90:
   move.l   D4,D2                ; temp->D5
   and.w    #square_size-1,D2    ; 
   move.l   #square_size,D5      ;
   sub.w    D2,D5                ;

   move.w   D4,D2                ; xb->D2
   asr.w    #_square,D2
   add.w    #1,D2
   asl.w    #1,D2

   move.l   D1,D3                ; yi->D3
   asl.w    #1,D3                ;
   move.w   D3,A6                ;
   mulu     0(A1,A6),D5          ;
   asr.l    #7,D5                ;
   move.w   D7,D3                ;
   sub.w    D5,D3                ;

   move.w   0(A3,A6),D6          ; y_step->D6
   tst.w    D3                   ;  check that yi>0
   bpl      iter_2               ;
   move     #dist_max,dist_x     ;
   bra      y_90                 ;

iter_2:                          
   move.w   D3,D5                ; 16*(yi/64)->D5
   asr.w    #_square,D5          ;
   asl.w    #_world+1,D5         ;
   add      D2,D5                ;
   move.w   0(A0,D5),D5
if_4:                            
   btst     #0,D5                ; if world(A5)<>0
   beq      else_4               ;
then_4:
   asr.w    #1,D2                ;
   asl.w    #_square,D2          ;
   sub.w    D4,D2                ; calculate distance_x_constant
   mulu     0(A4,A6),D2          ;
   asr.l    #7,D2                ;
   move.w   D2,dist_x            ;
   move.w   D3,yi                ;
   bra      y_90                 ;
else_4:
   add.w    #2,D2                ; ifnot calculation of yi and xb
   sub.w    D6,D3
   tst      D3                   ;  check yi>0
   bpl      iter_2
   move.w   #dist_max,dist_x
   bra      y_90
end_if_4:
end_if_x_90:

if_x_180:
   cmp      #ang_180,D1          ; if ang<180
   bhi      end_if_x_180
then_x_180:
   move.l   D4,D5                ; temp->D5
   and      #square_size-1,D5

   move.l   D4,D2                ; xb->D2
   asr.w    #_square,D2
   sub.w    #1,d2
   asl.w    #1,D2

   move.l   #ang_180,D3          ; yi->D3
   sub      D1,D3
   asl      #1,D3
   move.w   D3,A6
   mulu     0(A1,A6),D5
   asr.l    #7,D5
   move.w   D7,D3
   sub      D5,D3

   move     0(A3,A6),D6
   tst      D3                   ; check yi<0
   bpl      iter_3
   move     #dist_max,dist_x
   bra      y_180
iter_3:
   move.w   D3,D5                ; 16*(yi/64)->D5
   asr.w    #_square,D5          ;
   asl.w    #_world+1,D5
   add      D2,D5                ; 
   move.w   0(A0,D5),D5
if_6:                            
   btst     #0,D5                ; if world(A5)<>0
   beq      else_6
then_6:
   asr      #1,D2
   add      #1,D2
   asl      #_square,D2
   move     D4,D5
   sub      D2,D5                ; calculate distance_x_constant
   mulu     0(A4,A6),D5
   asr.l    #7,D5
   move     D5,dist_x
   move     D3,yi
   bra      y_180
else_6:
   sub      #2,D2
   sub      D6,D3
   tst      D3                    ; check yi<0
   bpl      iter_3
   move     #dist_max,dist_x
   bra      y_180
end_if_6:
end_if_x_180:


if_x_270:
   cmp      #ang_270,D1          ; if ang<270
   bhi      end_if_x_270
then_x_270:
   move.l   D4,D5                ; temp->D5
   and      #square_size-1,D5

   move.l   D4,D2                ; xb->D2
   asr.w    #_square,D2
   sub.w    #1,d2
   asl.w    #1,D2

   move.l   D1,D3                ;yi->D3
   sub      #ang_180,D3          
   asl      #1,D3
   move.w   D3,A6
   mulu     0(A1,A6),D5
   asr.l    #7,D5
   move.w   D7,D3
   add      D5,D3

   move     0(A3,A6),D6
   cmp      #world_size,D3       ; check yi>1024
   bls      iter_4
   move     #dist_max,dist_x
   bra      y_270
iter_4:
   move.w   D3,D5                ; 16*(yi/64)->D5
   asr.w    #_square,D5          ;
   asl.w    #_world+1,D5
   add      D2,D5                ; 
   move.w   0(A0,D5),D5
if_8:                            
   btst     #0,D5                ; if world(A5)<>0
   beq      else_8
then_8:
   asr      #1,D2
   add      #1,D2
   asl      #_square,D2
   move     D4,D5                ; calculate distance_x_constant
   sub      D2,D5
   mulu     0(A4,A6),D5
   asr.l    #7,D5
   move     D5,dist_x
   move     D3,yi
   bra      y_270
else_8:
   sub      #2,D2
   add      D6,D3
   cmp      #world_size,D3       ; check yi>1024
   bls      iter_4
   move     #dist_max,dist_x
   bra      y_270
end_if_8:
end_if_x_270:

;  si on est la, ang>270: alors, 
x_360:

   move.l   D4,D2                ; temp->D5
   and.w    #square_size-1,D2    ; 
   move.w   #square_size,D5      ;
   sub      D2,D5                ;

   move.l   D4,D2                ; xb->D2
   asr.w    #_square,D2
   add      #1,D2                ;
   asl      #1,D2

   move.l   #ang_360,D3          ; yi->D3
   sub      D1,D3                ; 
   asl.w    #1,D3                ;
   move.w   D3,A6                ;
   mulu     0(A1,A6),D5
   asr.l    #7,D5
   move.w   D7,D3                ;
   add      D5,D3
   move     0(A3,A6),D6
   cmp      #world_size,D3       ;  check yi>1024
   bls      iter_5
   move     #dist_max,dist_x
   bra      y_360
iter_5:
   move.l   D3,D5                ; 16*(yi/64)->D5
   asr.w    #_square,D5          ;
   asl.w    #_world+1,D5
   add      D2,D5                ; 
   move.w   0(A0,D5),D5
if_9:                            
   btst     #0,D5                ; if world(A5)<>0
   beq      else_9               ;
then_9:
   asr      #1,D2

   asl      #_square,D2
   sub      D4,D2
   mulu     0(A4,A6),D2          ; calculate distance_x_constant
   asr.l    #7,D2
   move.w   D2,dist_x
   move     D3,yi
   
   bra      y_360
else_9:
   add.w    #2,D2              ; ifnot calculation of yi and xb
   add.w    D6,D3              ;
   cmp      #world_size,D3     ; check yi>1024
   bls      iter_5
   move     #dist_max,dist_x
   bra      y_360
end_if_9:   

end_intersection_x_constant:


intersection_y_constant:

y_90:

   move.l   inv_tan_128_ptr,A1
   move.l   inv_tan_64_ptr,A3
   move.l   inv_sin_128_ptr,A4

   move.l   D7,D5                ; temp->D5
   and      #square_size-1,D5

   move.l   D7,D2                ; yb->D2
   asr.w    #_square,D2
   sub      #1,D2
   asl      #_world+1,D2

   move.l   D1,D3                ; xi->D3
   asl      #1,D3
   move.w   D3,A6
   mulu     0(A1,A6),D5
   asr.l    #7,D5
   move.w   D4,D3
   add      D5,D3
   move     0(A3,A6),D6
   cmp      #world_size,D3       ; check xi>1024
   bls      iter_6
   move     #dist_max,dist_y
   bra      render
iter_6:
   move.w   D3,D5                ; xi/64+1->D5
   asr.w    #_square,D5          ;
   asl      #1,D5                ;
   add      D2,D5                ; 
   move.w   0(A0,D5),D5
if_1_1:                            
   btst     #0,D5                ; if world(A5)<>0
   beq      else_1_1
then_1_1:
   asr      #_world+1,D2
   add      #1,D2
   asl      #_square,D2
   move     D7,D5                ; calculate distance_y_constant
   sub      D2,D5
   mulu     0(A4,A6),D5
   asr.l    #7,D5
   move     D5,dist_y
   move     D3,xi
   bra      render
else_1_1:
   sub      #2*_world_,D2
   add      D6,D3
   cmp      #world_size,D3       ; check xi>1024
   bls      iter_6
   move     #dist_max,dist_y
   bra      render
end_if_1_1:

y_180:

   move.l   inv_tan_128_ptr,A1
   move.l   inv_tan_64_ptr,A3
   move.l   inv_sin_128_ptr,A4

   move.l   D7,D5                ; temp->D5
   and      #square_size-1,D5

   move.l   D7,D2                ; yb->D2
   asr.w    #_square,D2
   sub      #1,D2
   asl      #_world+1,D2

   move.l   #ang_180,D3          ; xi->D3
   sub      D1,D3
   asl      #1,D3
   move.w   D3,A6
   mulu     0(A1,A6),D5
   asr.l    #7,D5
   move.w   D4,D3
   sub      D5,D3
   move     0(A3,A6),D6
   tst      D3                   ; check xi<0
   bpl      iter_7
   move     #dist_max,dist_y
   bra      render
iter_7:
   move.w   D3,D5                ; (xi/64+1)->D5
   asr.w    #_square,D5          ;
   asl      #1,D5                ;
   add      D2,D5                ; 
   move.w   0(A0,D5),D5
if_2_1:                            
   btst     #0,D5                ; if world(A5)<>0
   beq      else_2_1
then_2_1:
   asr      #_world+1,D2
   add      #1,D2
   asl      #_square,D2
   move     D7,D5
   sub      D2,D5                ; calculate distance_y_constant
   mulu     0(A4,A6),D5
   asr.l    #7,D5
   move     D5,dist_y
   move     D3,xi
   bra      render
else_2_1:
   sub      #2*_world_,D2
   sub      D6,D3
   tst      D3                   ; check xi<0
   bpl      iter_7
   move     #dist_max,dist_y
   bra      render
end_if_2_1:

y_270:

   move.l   inv_tan_128_ptr,A1
   move.l   inv_tan_64_ptr,A3
   move.l   inv_sin_128_ptr,A4

   move.l   D7,D2                ; temp->D5
   and.w    #square_size-1,D2    ; 
   move.l   #square_size,D5      ;
   sub      D2,D5                ;

   move.w   D7,D2                ; yb->D2
   asr.w    #_square,D2
   add      #1,D2
   asl      #_world+1,D2

   move.l   D1,D3                ; xi->D3
   sub      #ang_180,D3
   asl.w    #1,D3
   move.w   D3,A6
   mulu     0(A1,A6),D5
   asr.l    #7,D5
   move.w   D4,D3                
   sub      D5,D3                

   move     0(A3,A6),D6
   tst      D3                  ; check xi>0
   bpl      iter_8
   move     #dist_max,dist_y
   bra      render
iter_8:                          
   move.w   D3,D5                ; (xi/64+1)->D5
   asr.w    #_square,D5          ;
   asl      #1,D5                ;
   add      D2,D5                ; 
   move.w   0(A0,D5),D5
if_3_1:                            
   btst     #0,D5                ; if world(A5)<>0
   beq      else_3_1             ;
then_3_1:
   asr      #_world+1,D2
   asl      #_square,D2
   sub      D7,D2                ; calculate distance_y_constant
   mulu     0(A4,A6),D2
   asr.l    #7,D2
   move.w   D2,dist_y
   move     D3,xi
   bra      render
else_3_1:
   add.w    #2*_world_,D2        ; ifnot calculation of xi and yb
   sub      D6,D3
   tst      D3                   ; check xi>0
   bpl      iter_8
   move     #dist_max,dist_y
   bra      render
end_if_3_1:

y_360:

   move.l   inv_tan_128_ptr,A1
   move.l   inv_tan_64_ptr,A3
   move.l   inv_sin_128_ptr,A4

   move.l   D7,D2                ; temp->D5
   and.w    #square_size-1,D2    ; 
   move.l   #square_size,D5      ;
   sub      D2,D5                ;

   move.w   D7,D2                ; yb->D2
   asr.w    #_square,D2
   add      #1,D2
   asl      #_world+1,D2

   move.l   #ang_360,D3          ; xi->D3
   sub      D1,D3                ; 
   asl.w    #1,D3                ;
   move.w   D3,A6                ;
   mulu     0(A1,A6),D5
   asr.l    #7,D5
   move.w   D4,D3                ;
   add      D5,D3                ;

   move     0(A3,A6),D6
   cmp      #world_size,D3       ;  check xi>1024
   bls      iter_9
   move     #dist_max,dist_y
   bra      render
iter_9:                          
   move.w   D3,D5                ; (xi/64+1)->D5
   asr.w    #_square,D5          ;
   asl      #1,D5
   add      D2,D5                ; 
   move.w   0(A0,D5),D5
if_4_1:
   btst     #0,D5                ; if world(A5)<>0
   beq      else_4_1             ;
then_4_1:
   asr      #_world+1,D2
   asl      #_square,D2
   sub      D7,D2
   mulu     0(A4,A6),D2          ; calculate distance_y_constant
   asr.l    #7,D2
   move.w   D2,dist_y
   move     D3,xi
   bra      render
else_4_1:
   add.w    #_world_*2,D2        ; ifnot calculation of xi and yb
   add.w    D6,D3                ;
   cmp      #world_size,D3       ;  check xi>1024
   bls      iter_9
   move     #dist_max,dist_y
   bra      render
end_if_4_1:   

dist_max   = $FFFF

dist_x            dc.w 0
dist_y            dc.w 0
xi                dc.w 0
yi                dc.w 0
view_ang          dc.w 0

